@@ -4,7 +4,8 @@
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -262,6 +263,9 @@ grn_column_copy_same_table(grn_ctx *ctx,
   if (need_validate) {
     grn_obj validated_value;
     GRN_RECORD_INIT(&validated_value, to_range_flags, to_range_id);
+    if (to_range_flags & GRN_OBJ_WITH_WEIGHT) {
+      validated_value.header.flags |= GRN_OBJ_WITH_WEIGHT;
+    }
     bool is_uvector = grn_obj_is_uvector(ctx, &validated_value);
     grn_obj *range = grn_ctx_at(ctx, to_range_id);
     GRN_TABLE_EACH_BEGIN(ctx, table, cursor, id) {
