@@ -1,10 +1,11 @@
 /*
   Copyright(C) 2011-2018  Brazil
-  Copyright(C) 2018-2021  Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -67,7 +68,7 @@ class CriticalSection {
   CriticalSection &operator=(const CriticalSection &);
 };
 
-grn_inline static int
+inline static int
 grn_dat_name(grn_ctx *ctx, grn_dat *dat, char *buffer, int buffer_size)
 {
   int name_size;
@@ -707,89 +708,127 @@ class WALRecoverer {
     ctx_(ctx),
     dat_(dat),
     wal_error_tag_("[dat]"),
-    tag_("[dat][recover]") {
+    tag_("[dat][wal][recover]") {
   }
 
   grn_rc recover() {
-    if (!dat_->io) {
-      return GRN_SUCCESS;
-    }
-    if (dat_->io->path[0] == '\0') {
-      return GRN_SUCCESS;
+    if (ctx_->rc != GRN_SUCCESS) {
+      return ctx_->rc;
     }
 
     auto reader = grn_wal_reader_open(ctx_,
                                       reinterpret_cast<grn_obj *>(dat_),
                                       tag_);
-    if (!reader) {
+    if (ctx_->rc != GRN_SUCCESS) {
       return ctx_->rc;
     }
 
-    grn_io_clear_lock(dat_->io);
-
-    while (true) {
-      grn_wal_reader_entry entry = {};
-      grn_rc rc = grn_wal_reader_read_entry(ctx_, reader, &entry);
-      if (rc != GRN_SUCCESS) {
-        break;
-      }
-      switch (entry.event) {
-      case GRN_WAL_EVENT_ADD_ENTRY :
-        grn_dat_add_internal(ctx_,
-                             dat_,
-                             entry.key.content.binary.data,
-                             entry.key.content.binary.size,
-                             nullptr);
-        break;
-      case GRN_WAL_EVENT_DELETE_ENTRY :
-        if (!grn_dat_open_trie_if_needed(ctx_, dat_)) {
-          break;
-        }
-        if (entry.record_id == GRN_ID_NIL) {
-          grn_dat_delete_internal(ctx_,
-                                  dat_,
-                                  entry.key.content.binary.data,
-                                  entry.key.content.binary.size);
-        } else {
-          grn_dat_delete_by_id_internal(ctx_,
-                                        dat_,
-                                        entry.record_id);
-        }
-        break;
-      case GRN_WAL_EVENT_UPDATE_ENTRY :
-        if (entry.record_id == GRN_ID_NIL) {
-          grn_dat_update_internal(ctx_,
-                                  dat_,
-                                  entry.key.content.binary.data,
-                                  entry.key.content.binary.size,
-                                  entry.new_key.content.binary.data,
-                                  entry.new_key.content.binary.size);
-        } else {
-          grn_dat_update_by_id_internal(ctx_,
-                                        dat_,
-                                        entry.record_id,
-                                        entry.new_key.content.binary.data,
-                                        entry.new_key.content.binary.size);
-        }
-        break;
-      default :
-        grn_wal_set_recover_error(ctx_,
-                                  GRN_FUNCTION_NOT_IMPLEMENTED,
-                                  reinterpret_cast<grn_obj *>(dat_),
-                                  &entry,
-                                  wal_error_tag_,
-                                  "not implemented yet");
-        break;
-      }
-      if (ctx_->rc != GRN_SUCCESS) {
-        break;
-      }
-      grn_dat_wal_set_wal_id(ctx_, dat_, entry.id);
+    bool need_flush = false;
+    if (grn_io_is_locked(dat_->io)) {
+      grn_io_clear_lock(dat_->io);
+      need_flush = true;
     }
 
-    grn_wal_reader_close(ctx_, reader);
+    if (grn_dat_is_changing(ctx_, dat_)) {
+      grn_dat_clear_status_flags(ctx_, dat_);
+      need_flush = true;
+    }
 
-    if (ctx_->rc == GRN_SUCCESS) {
+    if (reader) {
+      while (true) {
+        grn_wal_reader_entry entry = {};
+        grn_rc rc = grn_wal_reader_read_entry(ctx_, reader, &entry);
+        if (rc != GRN_SUCCESS) {
+          break;
+        }
+        switch (entry.event) {
+        case GRN_WAL_EVENT_ADD_ENTRY :
+          grn_dat_add_internal(ctx_,
+                               dat_,
+                               entry.key.content.binary.data,
+                               entry.key.content.binary.size,
+                               nullptr);
+          break;
+        case GRN_WAL_EVENT_DELETE_ENTRY :
+          if (!grn_dat_open_trie_if_needed(ctx_, dat_)) {
+            break;
+          }
+          if (entry.record_id == GRN_ID_NIL) {
+            grn_dat_delete_internal(ctx_,
+                                    dat_,
+                                    entry.key.content.binary.data,
+                                    entry.key.content.binary.size);
+          } else {
+            grn_dat_delete_by_id_internal(ctx_,
+                                          dat_,
+                                          entry.record_id);
+          }
+          break;
+        case GRN_WAL_EVENT_UPDATE_ENTRY :
+          if (entry.record_id == GRN_ID_NIL) {
+            grn_dat_update_internal(ctx_,
+                                    dat_,
+                                    entry.key.content.binary.data,
+                                    entry.key.content.binary.size,
+                                    entry.new_key.content.binary.data,
+                                    entry.new_key.content.binary.size);
+          } else {
+            grn_dat_update_by_id_internal(ctx_,
+                                          dat_,
+                                          entry.record_id,
+                                          entry.new_key.content.binary.data,
+                                          entry.new_key.content.binary.size);
+          }
+          break;
+        default :
+          grn_wal_set_recover_error(ctx_,
+                                    GRN_FUNCTION_NOT_IMPLEMENTED,
+                                    reinterpret_cast<grn_obj *>(dat_),
+                                    &entry,
+                                    wal_error_tag_,
+                                    "not implemented yet");
+          break;
+        }
+        if (ctx_->rc != GRN_SUCCESS) {
+          break;
+        }
+        grn_dat_wal_set_wal_id(ctx_, dat_, entry.id);
+        need_flush = true;
+      }
+      grn_wal_reader_close(ctx_, reader);
+    }
+
+    if (ctx_->rc != GRN_SUCCESS) {
+      auto ctx = ctx_;
+      GRN_DEFINE_NAME(dat_);
+      GRN_LOG(ctx_, GRN_LOG_NOTICE,
+              "%s repair broken double array trie: <%.*s>(%u)",
+              tag_,
+              name_size,
+              name,
+              DB_OBJ(dat_)->id);
+      grn_rc repair_rc = grn_dat_repair(ctx_, dat_);
+      if (repair_rc == GRN_SUCCESS) {
+        ERRCLR(ctx_);
+        GRN_LOG(ctx_, GRN_LOG_NOTICE,
+                "%s succeeded to repair broken double array trie: <%.*s>(%u)",
+                tag_,
+                name_size,
+                name,
+                DB_OBJ(dat_)->id);
+        need_flush = true;
+      } else {
+        GRN_LOG(ctx_, GRN_LOG_ERROR,
+                "%s failed to recover double array trie: <%.*s>(%u): %s",
+                tag_,
+                name_size,
+                name,
+                DB_OBJ(dat_)->id,
+                ctx->errbuf);
+      }
+    }
+
+    if (need_flush && ctx_->rc == GRN_SUCCESS) {
       grn_obj_touch(ctx_, reinterpret_cast<grn_obj *>(dat_), NULL);
       grn_obj_flush(ctx_, reinterpret_cast<grn_obj *>(dat_));
     }
@@ -1628,6 +1667,19 @@ grn_dat_at(grn_ctx *ctx, grn_dat *dat, grn_id id)
   return id;
 }
 
+bool
+grn_dat_is_changing(grn_ctx *ctx, grn_dat *dat)
+{
+  if (!grn_dat_open_trie_if_needed(ctx, dat)) {
+    return false;
+  }
+  grn::dat::Trie * const trie = static_cast<grn::dat::Trie *>(dat->trie);
+  if (!trie) {
+    return false;
+  }
+  return trie->is_changing();
+}
+
 grn_rc
 grn_dat_clear_status_flags(grn_ctx *ctx, grn_dat *dat)
 {
@@ -1721,11 +1773,11 @@ grn_dat_dirty(grn_ctx *ctx, grn_dat *dat)
   return rc;
 }
 
-grn_bool
+bool
 grn_dat_is_dirty(grn_ctx *ctx, grn_dat *dat)
 {
   if (!dat->header) {
-    return GRN_FALSE;
+    return false;
   }
 
   return dat->header->n_dirty_opens > 0;
