@@ -1,10 +1,11 @@
 /*
   Copyright(C) 2011-2018  Brazil
-  Copyright(C) 2018-2021  Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -85,8 +86,10 @@ GRN_API grn_rc grn_dat_repair(grn_ctx *ctx, grn_dat *dat);
 
 GRN_API grn_rc grn_dat_flush(grn_ctx *ctx, grn_dat *dat);
 
+bool grn_dat_is_changing(grn_ctx *ctx, grn_dat *dat);
+
 grn_rc grn_dat_dirty(grn_ctx *ctx, grn_dat *dat);
-grn_bool grn_dat_is_dirty(grn_ctx *ctx, grn_dat *dat);
+bool grn_dat_is_dirty(grn_ctx *ctx, grn_dat *dat);
 grn_rc grn_dat_clean(grn_ctx *ctx, grn_dat *dat);
 grn_rc grn_dat_clear_dirty(grn_ctx *ctx, grn_dat *dat);
 
