@@ -1,10 +1,11 @@
 /*
   Copyright (C) 2009-2018  Brazil
-  Copyright (C) 2019-2022  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2019-2024  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,8 +33,6 @@ void
 grn_proc_init_from_env(void);
 void
 grn_proc_query_init_from_env(void);
-void
-grn_proc_select_init_from_env(void);
 
 GRN_VAR const char *grn_document_root;
 void
@@ -64,6 +63,14 @@ grn_proc_init_config_delete(grn_ctx *ctx);
 void
 grn_proc_init_define_selector(grn_ctx *ctx);
 void
+grn_proc_init_distance_cosine(grn_ctx *ctx);
+void
+grn_proc_init_distance_inner_product(grn_ctx *ctx);
+void
+grn_proc_init_distance_l1_norm(grn_ctx *ctx);
+void
+grn_proc_init_distance_l2_norm_squared(grn_ctx *ctx);
+void
 grn_proc_init_dump(grn_ctx *ctx);
 void
 grn_proc_init_edit_distance(grn_ctx *ctx);
@@ -156,14 +163,18 @@ grn_proc_init_thread_limit(grn_ctx *ctx);
 void
 grn_proc_init_tokenize(grn_ctx *ctx);
 
-grn_bool
-grn_proc_option_value_bool(grn_ctx *ctx,
-                           grn_obj *option,
-                           grn_bool default_value);
+bool
+grn_proc_option_value_bool(grn_ctx *ctx, grn_obj *option, bool default_value);
 int32_t
 grn_proc_option_value_int32(grn_ctx *ctx,
                             grn_obj *option,
                             int32_t default_value);
+uint32_t
+grn_proc_option_value_uint32(grn_ctx *ctx,
+                             grn_obj *option,
+                             uint32_t default_value);
+float
+grn_proc_option_value_float(grn_ctx *ctx, grn_obj *option, float default_value);
 double
 grn_proc_option_value_double(grn_ctx *ctx,
                              grn_obj *option,
@@ -237,7 +248,7 @@ grn_proc_output_object_name(grn_ctx *ctx, grn_obj *obj);
 void
 grn_proc_output_object_id_name(grn_ctx *ctx, grn_id id);
 
-grn_bool
+bool
 grn_proc_table_set_token_filters(grn_ctx *ctx,
                                  grn_obj *table,
                                  grn_raw_string *token_filters_raw);
@@ -278,8 +289,7 @@ grn_proc_syntax_expand_query(grn_ctx *ctx,
                              const char *query,
                              unsigned int query_len,
                              grn_expr_flags flags,
-                             const char *query_expander_name,
-                             unsigned int query_expander_name_len,
+                             grn_obj *query_expander,
                              const char *term_column_name,
                              unsigned int term_column_name_len,
                              const char *expanded_term_column_name,
@@ -299,7 +309,7 @@ grn_proc_lexicon_open(grn_ctx *ctx,
                       grn_raw_string *token_filters_raw,
                       const char *context_tag);
 
-grn_bool
+bool
 grn_proc_text_include_special_character(grn_ctx *ctx,
                                         const char *text,
                                         size_t size);
