@@ -1,10 +1,11 @@
 /*
-  Copyright(C) 2018  Brazil
-  Copyright(C) 2019-2021  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2018  Brazil
+  Copyright (C) 2019-2024  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -47,6 +48,9 @@ grn_table_setoperation_with_weight_factor(grn_ctx *ctx,
                                           grn_operator op,
                                           float weight_factor);
 
+grn_id
+grn_table_find_reference_object_raw(grn_ctx *ctx, grn_id table_id);
+
 grn_hash *
 grn_table_all_columns(grn_ctx *ctx, grn_obj *table);
 
@@ -62,10 +66,10 @@ grn_table_all_columns(grn_ctx *ctx, grn_obj *table);
     }                                                                   \
   }                                                                     \
   if (can_run_) {
-#define GRN_TABLE_LOCK_END(ctx, table)          \
+#define GRN_TABLE_LOCK_END(ctx)                 \
   }                                             \
   if (locked_) {                                \
-    grn_io_unlock(io_);                         \
+    grn_io_unlock(ctx, io_);                    \
   }                                             \
 } while (GRN_FALSE)
 
