@@ -1,10 +1,11 @@
 /*
   Copyright(C) 2009-2018  Brazil
-  Copyright(C) 2018-2020  Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,7 +36,7 @@ grn_output_columns_parse(grn_ctx *ctx,
     grn_expr_parse(ctx,
                    output_columns,
                    raw_output_columns,
-                   raw_output_columns_size,
+                   (unsigned int)raw_output_columns_size,
                    NULL,
                    GRN_OP_MATCH,
                    GRN_OP_AND,
@@ -54,9 +55,9 @@ grn_output_columns_get_offsets(grn_ctx *ctx,
                                grn_obj *output_columns,
                                grn_obj *offsets)
 {
-  int previous_comma_offset = -1;
-  grn_bool is_first_comma = GRN_TRUE;
-  grn_bool have_comma = GRN_FALSE;
+  ptrdiff_t previous_comma_offset = -1;
+  bool is_first_comma = true;
+  bool have_comma = false;
   grn_expr *expr = (grn_expr *)output_columns;
   grn_expr_code *code;
   grn_expr_code *code_end = expr->codes + expr->codes_curr;
@@ -64,23 +65,23 @@ grn_output_columns_get_offsets(grn_ctx *ctx,
   GRN_API_ENTER;
 
   for (code = expr->codes; code < code_end; code++) {
-    int code_start_offset;
+    ptrdiff_t code_start_offset;
 
     if (code->op != GRN_OP_COMMA) {
       continue;
     }
 
-    have_comma = GRN_TRUE;
+    have_comma = true;
     if (is_first_comma) {
       unsigned int n_used_codes;
-      int code_end_offset;
+      ptrdiff_t code_end_offset;
 
       n_used_codes = grn_expr_code_n_used_codes(ctx, expr->codes, code - 1);
       code_end_offset = code - expr->codes - n_used_codes;
       GRN_UINT32_PUT(ctx, offsets, 0);
       GRN_UINT32_PUT(ctx, offsets, code_end_offset);
       code_start_offset = code_end_offset;
-      is_first_comma = GRN_FALSE;
+      is_first_comma = false;
     } else {
       code_start_offset = previous_comma_offset + 1;
     }
@@ -228,7 +229,7 @@ grn_output_columns_apply_add_records_key(grn_ctx *ctx,
 
   n_columns = GRN_BULK_VSIZE(columns) / sizeof(grn_obj *);
   {
-    grn_bool have_key = GRN_FALSE;
+    bool have_key = false;
     size_t i;
     for (i = 0; i < n_columns; i++) {
       grn_obj *column = GRN_PTR_VALUE_AT(columns, i);
@@ -248,7 +249,7 @@ grn_output_columns_apply_add_records_key(grn_ctx *ctx,
           goto exit;
         }
         key_offset = i;
-        have_key = GRN_TRUE;
+        have_key = true;
       }
     }
 
@@ -298,7 +299,7 @@ grn_output_columns_apply_add_records_key(grn_ctx *ctx,
       target_id = grn_table_add(ctx,
                                 target_table,
                                 GRN_BULK_HEAD(&target_key_buffer),
-                                GRN_BULK_VSIZE(&target_key_buffer),
+                                (unsigned int)GRN_BULK_VSIZE(&target_key_buffer),
                                 NULL);
       if (target_id == GRN_ID_NIL) {
         continue;
