@@ -1,10 +1,11 @@
 /*
   Copyright (C) 2012-2018  Brazil
-  Copyright (C) 2022  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2022-2024  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1091,7 +1092,7 @@ grn_plugin_get_names(grn_ctx *ctx, grn_obj *names)
   const char *system_plugins_dir;
   const char *native_plugin_suffix;
   const char *ruby_plugin_suffix;
-  grn_bool is_close_opened_object_mode = GRN_FALSE;
+  bool is_close_opened_object_mode = false;
 
   GRN_API_ENTER;
 
@@ -1100,7 +1101,7 @@ grn_plugin_get_names(grn_ctx *ctx, grn_obj *names)
   }
 
   if (grn_thread_get_limit() == 1) {
-    is_close_opened_object_mode = GRN_TRUE;
+    is_close_opened_object_mode = true;
   }
 
   processed_paths = grn_hash_create(ctx, NULL, GRN_TABLE_MAX_KEY_SIZE, 0,
@@ -1369,12 +1370,12 @@ grn_plugin_proc_get_var(grn_ctx *ctx, grn_user_data *user_data,
   return grn_proc_get_var(ctx, user_data, name, (unsigned int)name_size);
 }
 
-grn_bool
+bool
 grn_plugin_proc_get_var_bool(grn_ctx *ctx,
                              grn_user_data *user_data,
                              const char *name,
                              int name_size,
-                             grn_bool default_value)
+                             bool default_value)
 {
   grn_obj *var;
 
@@ -1395,6 +1396,19 @@ grn_plugin_proc_get_var_int32(grn_ctx *ctx,
   return grn_proc_option_value_int32(ctx, var, default_value);
 }
 
+double
+grn_plugin_proc_get_var_double(grn_ctx *ctx,
+                               grn_user_data *user_data,
+                               const char *name,
+                               int name_size,
+                               double default_value)
+{
+  grn_obj *var;
+
+  var = grn_plugin_proc_get_var(ctx, user_data, name, name_size);
+  return grn_proc_option_value_double(ctx, var, default_value);
+}
+
 const char *
 grn_plugin_proc_get_var_string(grn_ctx *ctx,
                                grn_user_data *user_data,
@@ -1543,6 +1557,15 @@ grn_plugin_proc_get_value_bool(grn_ctx *ctx,
   return grn_proc_get_value_bool(ctx, value, default_value, tag);
 }
 
+int32_t
+grn_plugin_proc_get_value_int32(grn_ctx *ctx,
+                                grn_obj *value,
+                                int32_t default_value_raw,
+                                const char *tag)
+{
+  return grn_proc_get_value_int32(ctx, value, default_value_raw, tag);
+}
+
 int64_t
 grn_plugin_proc_get_value_int64(grn_ctx *ctx,
                                 grn_obj *value,
@@ -1552,6 +1575,15 @@ grn_plugin_proc_get_value_int64(grn_ctx *ctx,
   return grn_proc_get_value_int64(ctx, value, default_value_raw, tag);
 }
 
+double
+grn_plugin_proc_get_value_double(grn_ctx *ctx,
+                                 grn_obj *value,
+                                 double default_value_raw,
+                                 const char *tag)
+{
+  return grn_proc_get_value_double(ctx, value, default_value_raw, tag);
+}
+
 grn_operator
 grn_plugin_proc_get_value_mode(grn_ctx *ctx,
                                grn_obj *value,
