@@ -1,9 +1,11 @@
 /*
-  Copyright(C) 2015 Brazil
+  Copyright(C) 2015  Brazil
+  Copyright(C) 2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,9 +28,9 @@ scorer_tf_idf(grn_ctx *ctx, grn_scorer_matched_record *record)
 {
   double min_score = 1.0;
   double tf;
-  double n_all_documents;
-  double n_candidates;
-  double n_tokens;
+  uint64_t n_all_documents;
+  uint64_t n_candidates;
+  uint32_t n_tokens;
   double n_estimated_match_documents;
 
   tf = grn_scorer_matched_record_get_n_occurrences(ctx, record) +
@@ -36,15 +38,15 @@ scorer_tf_idf(grn_ctx *ctx, grn_scorer_matched_record *record)
   n_all_documents = grn_scorer_matched_record_get_n_documents(ctx, record);
   n_candidates = grn_scorer_matched_record_get_n_candidates(ctx, record);
   n_tokens = grn_scorer_matched_record_get_n_tokens(ctx, record);
-  n_estimated_match_documents = n_candidates / n_tokens;
+  n_estimated_match_documents = (double)n_candidates / (double)n_tokens;
 
-  if (n_estimated_match_documents >= n_all_documents) {
+  if (n_estimated_match_documents >= (double)n_all_documents) {
     return min_score;
   } else {
     double idf;
     double tf_idf;
 
-    idf = log(n_all_documents / n_estimated_match_documents);
+    idf = log((double)n_all_documents / n_estimated_match_documents);
     tf_idf = tf * idf;
     return fmax(tf_idf, min_score);
   }
