@@ -1,10 +1,11 @@
 /*
-  Copyright(C) 2017-2018  Brazil
-  Copyright(C) 2018-2021  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2017-2018  Brazil
+  Copyright (C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -407,7 +408,7 @@ grn_table_get_module_string_raw(grn_ctx *ctx,
                                 const char *module_name)
 {
   char name[GRN_TABLE_MAX_KEY_SIZE];
-  unsigned int name_size;
+  int name_size;
   grn_obj options;
   unsigned int n = 0;
 
