@@ -1,10 +1,11 @@
 /*
   Copyright(C) 2016-2017  Brazil
-  Copyright(C) 2019-2021  Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2019-2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,8 +39,8 @@ grn_window_shard_init(grn_ctx *ctx,
   grn_expr *expr = (grn_expr *)(window_function_call);
   shard->window_function = (grn_proc *)(expr->codes[0].value);
   GRN_PTR_INIT(&(shard->arguments), GRN_OBJ_VECTOR, GRN_ID_NIL);
-  int32_t n_arguments = expr->codes_curr - 1;
-  for (int32_t i = 1; i < n_arguments; i++) {
+  uint32_t n_arguments = expr->codes_curr - 1;
+  for (uint32_t i = 1; i < n_arguments; i++) {
     /* TODO: Check op. */
     GRN_PTR_PUT(ctx, &(shard->arguments), expr->codes[i].value);
   }
@@ -173,10 +174,11 @@ grn_window_rewind(grn_ctx *ctx, grn_window *window)
       shard->current_index = 0;
     }
   } else {
-    window->current_shard = window->n_shards - 1;
+    window->current_shard = (ssize_t)(window->n_shards - 1);
     for (size_t i = 0; i < window->n_shards; i++) {
       grn_window_shard *shard = &(window->shards[i]);
-      shard->current_index = GRN_RECORD_VECTOR_SIZE(&(shard->ids)) - 1;
+      shard->current_index =
+        (ssize_t)(GRN_RECORD_VECTOR_SIZE(&(shard->ids)) - 1);
     }
   }
 
@@ -384,7 +386,7 @@ grn_window_reset(grn_ctx *ctx,
   GRN_API_RETURN(ctx->rc);
 }
 
-static grn_bool
+static bool
 grn_expr_is_window_function_call(grn_ctx *ctx,
                                  grn_obj *window_function_call)
 {
@@ -396,20 +398,20 @@ grn_expr_is_window_function_call(grn_ctx *ctx,
   call = &(expr->codes[expr->codes_curr - 1]);
 
   if (func->op != GRN_OP_PUSH) {
-    return GRN_FALSE;
+    return false;
   }
   if (!grn_obj_is_window_function_proc(ctx, func->value)) {
-    return GRN_FALSE;
+    return false;
   }
 
   if (call->op != GRN_OP_CALL) {
-    return GRN_FALSE;
+    return false;
   }
   if (call->nargs != (int32_t)(expr->codes_curr - 1)) {
-    return GRN_FALSE;
+    return false;
   }
 
-  return GRN_TRUE;
+  return true;
 }
 
 static bool
@@ -652,7 +654,7 @@ grn_window_function_create(grn_ctx *ctx,
   GRN_API_ENTER;
 
   if (name_size == -1) {
-    name_size = strlen(name);
+    name_size = (int)strlen(name);
   }
 
   window_function = grn_proc_create(ctx,
@@ -706,7 +708,7 @@ grn_window_execute(grn_ctx *ctx, grn_window *window)
                                    shard->output_column,
                                    window,
                                    (grn_obj **)GRN_BULK_HEAD(&(shard->arguments)),
-                                   GRN_PTR_VECTOR_SIZE(&(shard->arguments)));
+                                   (int)GRN_PTR_VECTOR_SIZE(&(shard->arguments)));
 
   GRN_API_RETURN(rc);
 }
@@ -778,7 +780,7 @@ grn_table_apply_window_function(grn_ctx *ctx,
                  table,
                  0, -1,
                  sorted,
-                 sort_keys, n_sort_keys);
+                 sort_keys, (int)n_sort_keys);
 
   grn_window window;
   grn_window_init(ctx, &window);
