@@ -1,10 +1,11 @@
 /*
   Copyright (C) 2015-2018  Brazil
-  Copyright (C) 2022  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2022-2023  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -210,6 +211,9 @@ func_vector_slice(grn_ctx *ctx, int n_args, grn_obj **args,
 
   if (target->header.flags & GRN_OBJ_WITH_WEIGHT) {
     slice->header.flags |= GRN_OBJ_WITH_WEIGHT;
+    if (target->header.flags & GRN_OBJ_WEIGHT_BFLOAT16) {
+      slice->header.flags |= GRN_OBJ_WEIGHT_BFLOAT16;
+    }
   }
 
   if (length < 0) {
